@@ -18,6 +18,7 @@ from .constants import AMPLITUDE_UUID
 from .constants import AUC_UUID
 from .constants import CENTIMILLISECONDS_PER_SECOND
 from .constants import MIN_NUMBER_PEAKS
+from .constants import MIN_NUMBER_VALLEYS
 from .constants import PRIOR_PEAK_INDEX_UUID
 from .constants import PRIOR_VALLEY_INDEX_UUID
 from .constants import SUBSEQUENT_PEAK_INDEX_UUID
@@ -30,12 +31,14 @@ from .constants import WIDTH_UUID
 from .constants import WIDTH_VALUE_UUID
 from .exceptions import TooFewPeaksDetectedError
 from .exceptions import TwoPeaksInARowError
+from .exceptions import TwoValleysInARowError
 
 TWITCH_WIDTH_PERCENTS = range(10, 95, 5)
 
 
 def peak_detector(
-    filtered_gmr: NDArray[(2, Any), int], twitches_point_up: bool = True,
+    filtered_gmr: NDArray[(2, Any), int],
+    twitches_point_up: bool = True,
 ) -> Tuple[List[int], List[int]]:
     """Locates peaks and valleys and returns the indices.
 
@@ -55,8 +58,8 @@ def peak_detector(
         peak_invertor_factor *= -1
         valley_invertor_factor *= -1
     sampling_period_cms = filtered_gmr[0, 1] - filtered_gmr[0, 0]
-    maximum_possible_twitch_frequency = 7  # pylint:disable=invalid-name # (Eli 9/1/20: I can't think of a shorter name to describe this concept fully) # Hz
-    minimum_required_samples_between_twitches = int(  # pylint:disable=invalid-name # (Eli 9/1/20: I can't think of a shorter name to describe this concept fully)
+    maximum_possible_twitch_frequency = 7  # pylint:disable=invalid-name # (Eli 9/1/20): I can't think of a shorter name to describe this concept fully # Hz
+    minimum_required_samples_between_twitches = int(  # pylint:disable=invalid-name # (Eli 9/1/20): I can't think of a shorter name to describe this concept fully
         round(
             (1 / maximum_possible_twitch_frequency)
             * CENTIMILLISECONDS_PER_SECOND
@@ -69,20 +72,30 @@ def peak_detector(
     max_height = np.max(gmr_signal)
     min_height = np.min(gmr_signal)
     max_prominence = abs(max_height - min_height)
-    # print(f"max prom {max_prominence}")
     # find peaks and valleys
     peak_indices, _ = signal.find_peaks(
         gmr_signal * peak_invertor_factor,
         width=minimum_required_samples_between_twitches / 2,
         distance=minimum_required_samples_between_twitches,
-        prominence=max_prominence / 22,
+        prominence=max_prominence / 4,
     )
-    valley_indices, _ = signal.find_peaks(
+    valley_indices, properties = signal.find_peaks(
         gmr_signal * valley_invertor_factor,
         width=minimum_required_samples_between_twitches / 2,
         distance=minimum_required_samples_between_twitches,
-        prominence=max_prominence / 22,
+        prominence=max_prominence / 4,
     )
+    left_valley_bases = properties["left_bases"]
+    right_valley_bases = properties["right_bases"]
+    # TODO Tanner (11/3/20): move this loop to find_twitch_indices
+    for i in range(1, len(valley_indices)):
+        if (
+            left_valley_bases[i] == left_valley_bases[i - 1]
+            and right_valley_bases[i] == right_valley_bases[i - 1]
+        ):
+            valley_indices = np.delete(valley_indices, i)
+            i -= 1
+
     return peak_indices, valley_indices
 
 
@@ -268,33 +281,71 @@ def find_twitch_indices(
         raise TooFewPeaksDetectedError(
             f"A minimum of {MIN_NUMBER_PEAKS} peaks is required to extract twitch metrics, however only {len(peak_indices)} peak(s) were detected"
         )
+    if len(valley_indices) < MIN_NUMBER_VALLEYS:
+        raise TooFewPeaksDetectedError(
+            f"A minimum of {MIN_NUMBER_VALLEYS} valleys is required to extract twitch metrics, however only {len(valley_indices)} valley(s) were detected"
+        )
     twitches: Dict[int, Dict[UUID, Optional[int]]] = {}
+
     starts_with_peak = peak_indices[0] < valley_indices[0]
-    # ends_with_peak = peak_indices[-1] > valley_indices[-1]
+    prev_feature_is_peak = starts_with_peak
+    peak_idx = 0
+    valley_idx = 0
+    if starts_with_peak:
+        peak_idx += 1
+    else:
+        valley_idx += 1
+
+    # check for two back-to-back features
+    while peak_idx < len(peak_indices) and valley_idx < len(valley_indices):
+        if prev_feature_is_peak:
+            if valley_indices[valley_idx] > peak_indices[peak_idx]:
+                raise TwoPeaksInARowError(
+                    peak_and_valley_indices,
+                    filtered_data,
+                    (peak_indices[peak_idx - 1], peak_indices[peak_idx]),
+                )
+            valley_idx += 1
+        else:
+            if valley_indices[valley_idx] < peak_indices[peak_idx]:
+                raise TwoValleysInARowError(
+                    peak_and_valley_indices,
+                    filtered_data,
+                    (valley_indices[valley_idx - 1], valley_indices[valley_idx]),
+                )
+            peak_idx += 1
+        prev_feature_is_peak = not prev_feature_is_peak
+    if peak_idx < len(peak_indices) - 1:
+        raise TwoPeaksInARowError(
+            peak_and_valley_indices,
+            filtered_data,
+            (peak_indices[peak_idx], peak_indices[peak_idx + 1]),
+        )
+    if valley_idx < len(valley_indices) - 1:
+        raise TwoValleysInARowError(
+            peak_and_valley_indices,
+            filtered_data,
+            (valley_indices[valley_idx], valley_indices[valley_idx + 1]),
+        )
+
+    # compile dict of twitch information
     for itr_idx, itr_peak_index in enumerate(peak_indices):
-        if itr_idx == peak_indices.shape[0] - 1:  # last peak
-            continue
+        if itr_idx < peak_indices.shape[0] - 1:  # last peak
+            if itr_idx == 0 and starts_with_peak:
+                continue
 
-        if valley_indices[itr_idx] > peak_indices[itr_idx + 1]:
-            raise TwoPeaksInARowError(
-                peak_and_valley_indices,
-                filtered_data,
-                (peak_indices[itr_idx], peak_indices[itr_idx + 1]),
-            )
-
-        if itr_idx == 0 and starts_with_peak:
-            continue
-
-        twitches[itr_peak_index] = {
-            PRIOR_PEAK_INDEX_UUID: None if itr_idx == 0 else peak_indices[itr_idx - 1],
-            PRIOR_VALLEY_INDEX_UUID: valley_indices[
-                itr_idx - 1 if starts_with_peak else itr_idx
-            ],
-            SUBSEQUENT_PEAK_INDEX_UUID: peak_indices[itr_idx + 1],
-            SUBSEQUENT_VALLEY_INDEX_UUID: valley_indices[
-                itr_idx if starts_with_peak else itr_idx + 1
-            ],
-        }
+            twitches[itr_peak_index] = {
+                PRIOR_PEAK_INDEX_UUID: None
+                if itr_idx == 0
+                else peak_indices[itr_idx - 1],
+                PRIOR_VALLEY_INDEX_UUID: valley_indices[
+                    itr_idx - 1 if starts_with_peak else itr_idx
+                ],
+                SUBSEQUENT_PEAK_INDEX_UUID: peak_indices[itr_idx + 1],
+                SUBSEQUENT_VALLEY_INDEX_UUID: valley_indices[
+                    itr_idx if starts_with_peak else itr_idx + 1
+                ],
+            }
 
     # print(list(twitches.keys())[0])
     # print(twitches[list(twitches.keys())[0]])
